/* Software Interrupt handling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#include "exception.h"
#include "exceptions.h"
#include "ledcycle.h"
#include "backtrace.h"
#include <os_arm_bare_registers_def.h>
#include <stdio.h>

static void printFault(uint32_t lr, const uint32_t * fp, 
	const struct _ExcProcessorStatus * pstat, uint32_t swi);

__attribute__((interrupt("SWI"),weak))
void hplarmv_swi() {
	/* gather some data: lr, fp, processor status */
	uint32_t lr = (uint32_t)__builtin_return_address(0);
	uint32_t * fp = (uint32_t*)__builtin_frame_address(0);
	struct _ExcProcessorStatus pstat;
	_excGetProcessorStatus(&pstat);
	
	uint32_t swi = ((uint32_t*)lr)[-1] & 0x00ffffff;
	
	/* initialize exception handler */
	_excInit();
	
	/* begin in next line */
	_excUartOut("\n");
	
	while (1) {
		printFault(lr, fp, &pstat, swi);
		_excLedCycle(TRUE);
	}
}

static void printFault(uint32_t lr, const uint32_t * fp,
	const struct _ExcProcessorStatus * pstat, uint32_t swi) {
	char buf[128];
	
	/* print the exception type and the old mode */
	snprintf(buf, sizeof(buf), "Software Interrupt Request %d"
		" (Not implemented!) in %s mode\n", swi, pstat->spsrModeLabel);
	_excUartOut(buf);
	
	_excPrintProcessorStatus(pstat->oldlr, fp, pstat);
	
	_excBacktrace(lr, fp);
}

